@@ -1,6 +1,6 @@
 /*
  *  Methods for encryption/security mechanisms for cryptmount
- *  (C)Copyright 2005-2013, RW Penney
+ *  (C)Copyright 2005-2014, RW Penney
  */
 
 /*
@@ -784,6 +784,8 @@ int sycheck_target(const tgtdefn_t *tgt)
  *  ==== Mutex-locking on configuration directory ====
  */
 
+static const char *cm_lock_filename = "_cryptmount_lock_";
+
 int cm_mutex_lock(void)
     /** Try to acquire lock on configuration directory (via symlink marker) */
 {   char *fname=NULL, ident[64];
@@ -793,7 +795,7 @@ int cm_mutex_lock(void)
     struct timespec delay;
 #endif
 
-    (void)cm_path(&fname, "_lock_");
+    (void)cm_path(&fname, CM_SYSRUN_PFX, cm_lock_filename);
     snprintf(ident, sizeof(ident), "%u-%u",
                 (unsigned)getpid(), (unsigned)getuid());
 
@@ -828,13 +830,17 @@ int cm_mutex_lock(void)
     return eflag;
 }
 
+/**
+ *  Release an inter-process lock on configuration directory.
+ *
+ *  \see cm_mutex_lock().
+ */
 int cm_mutex_unlock(void)
-    /** Release lock on configuration directory */
 {   char *fname=NULL;
     struct stat sbuff;
     int eflag=0;
 
-    (void)cm_path(&fname, "_lock_");
+    (void)cm_path(&fname, CM_SYSRUN_PFX, cm_lock_filename);
 
     if (lstat(fname, &sbuff) != 0
       || !S_ISLNK(sbuff.st_mode)
@@ -851,5 +857,5 @@ int cm_mutex_unlock(void)
 /** @} */
 
 /*
- *  (C)Copyright 2005-2013, RW Penney
+ *  (C)Copyright 2005-2014, RW Penney
  */
