@@ -1,6 +1,6 @@
 /*
  *  Methods for unit-testing utiltities for cryptmount
- *  (C)Copyright 2006-2013, RW Penney
+ *  (C)Copyright 2006-2014, RW Penney
  */
 
 /*
@@ -75,6 +75,12 @@ int cm_tests_close()
 int cm_run_tests()
     /* Front-end to self-testing routines */
 {
+    int bf_test_blowfish(),
+        fs_test_blkgetsz(), fs_test_splitopts(), fs_test_entropy(),
+        km_test_managers(), km_test_keyrw(), km_test_legacy(),
+        tb_test_expand(),
+        ut_test_strings(), ut_test_strops(), ut_test_sha1(), ut_pwfort();
+
     cm_tests_init();
 
     ut_test_strings();
@@ -101,5 +107,5 @@ int _keep_ansi_pedantic_quiet = 0;
 #endif  /* TESTING */
 
 /*
- *  (C)Copyright 2006-2013, RW Penney
+ *  (C)Copyright 2006-2014, RW Penney
  */
