@@ -1,6 +1,6 @@
 /*
  *  cryptmount - a utility for user-level mounting of encrypted filesystems
- *  (C)Copyright 2005-2013, RW Penney
+ *  (C)Copyright 2005-2014, RW Penney
  */
 
 /*
@@ -278,7 +278,7 @@ static int do_keymgrlist()
 
 
 static int do_sysboot_updown(int booting, const tgtdefn_t *tgttable,
-                            const km_pw_context_t *pw_ctxt)
+                             const km_pw_context_t *pw_ctxt)
     /** Setup all devices which have a bootaction option specified */
 {   const tgtdefn_t *tgt;
     bound_tgtdefn_t *boundtgt = NULL;
@@ -316,17 +316,17 @@ static int do_sysboot_updown(int booting, const tgtdefn_t *tgttable,
 
 
 static int do_devsetup(const km_pw_context_t *pw_ctxt,
-                    bound_tgtdefn_t *boundtgt, char **mntdev)
+                       bound_tgtdefn_t *boundtgt, char **mntdev)
     /** Setup all devices needed to access encrypted target */
 {   enum { BUFFMIN=1024 };
-    unsigned char *key=NULL;
-    int i, blklen, readonly, isloop = 0, killloop = 0,
+    unsigned char *key = NULL;
+    int buffpos, blklen, readonly, isloop = 0, killloop = 0,
         keylen = 0, eflag = ERR_NOERROR;
     int64_t devlen = 0, fslen = 0;
     size_t dpsize;
-    char *dmparams=NULL;
-    const char *tgtdev=NULL;
-    const tgtdefn_t *tgt=NULL;
+    char *dmparams = NULL;
+    const char *tgtdev = NULL;
+    const tgtdefn_t *tgt = NULL;
 
     /* Get crypto-key for filing system: */
     eflag = cm_get_key(boundtgt, pw_ctxt, &key, &keylen);
@@ -368,11 +368,17 @@ static int do_devsetup(const km_pw_context_t *pw_ctxt,
     /* Setup device-mapper crypt table (CIPHER KEY IV_OFFSET DEV START): */
     dpsize = 2 * keylen + BUFFMIN;
     dmparams = sec_realloc(dmparams, dpsize);
-    i = snprintf(dmparams, dpsize, "%s ",
-                (tgt->cipher != NULL ? tgt->cipher : DFLT_CIPHER));
-    i += mk_key_string(key, (size_t)keylen, dmparams + i);
-    snprintf(dmparams + i, (dpsize - i), " %" PRId64 " %s %" PRId64,
-                tgt->ivoffset, tgtdev, tgt->start);
+    buffpos = snprintf(dmparams, dpsize, "%s ",
+                        (tgt->cipher != NULL ?  tgt->cipher
+                                             : CM_DEFAULT_CIPHER));
+    buffpos += mk_key_string(key, (size_t)keylen, dmparams + buffpos);
+    buffpos += snprintf(dmparams + buffpos, (dpsize - buffpos),
+                          " %" PRId64 " %s %" PRId64,
+                          tgt->ivoffset, tgtdev, tgt->start);
+    if ((tgt->flags & FLG_TRIM) != 0) {
+      buffpos += snprintf(dmparams + buffpos, (dpsize - buffpos),
+                            " 1 allow_discards");
+    }
 
     /* Setup device-mapper target: */
     eflag = devmap_create(tgt->ident,
@@ -860,7 +866,7 @@ static int do_safetynet()
         tgt->ident = tst->ident;
 
 #ifdef DLGT_UMOUNT
-        /* attempt to unmount filesystem: */
+        /* Attempt to unmount filesystem: */
         switch (fork()) {
             case -1:
                 break;
@@ -901,6 +907,16 @@ static int do_safetynet()
 
     free_tgtstatus(all_tsts);
 
+    if (!is_cmstatus_intact()) {
+      char *statpath = NULL;
+      (void)cm_path(&statpath, CM_SYSRUN_PFX, cm_status_filename);
+      if (statpath != NULL) {
+        fprintf(stderr, "%s has been corrupted - removing\n", statpath);
+        unlink(statpath);
+        free(statpath);
+      }
+    }
+
     tgt->ident = old_ident;
     free_tgtdefn(tgt);
 
@@ -1311,7 +1327,7 @@ int main(int argc, char *argv[])
     }
 #endif
 
-    (void)cm_path(&cmtab, "cmtab");
+    (void)cm_path(&cmtab, CM_SYSCONF_PFX, "cmtab");
 
     if (mode == M_HELP) {
         fprintf(stderr, "%s", _(USAGE_STRING));
@@ -1412,5 +1428,5 @@ int main(int argc, char *argv[])
 }
 
 /*
- *  (C)Copyright 2005-2013, RW Penney
+ *  (C)Copyright 2005-2014, RW Penney
  */
