@@ -1,6 +1,6 @@
 /*
  *  Miscellaneous utility functions for cryptmount
- *  (C)Copyright 2005-2013, RW Penney
+ *  (C)Copyright 2005-2014, RW Penney
  */
 
 /*
@@ -181,17 +181,38 @@ int ut_test_strings()
 #endif  /* TESTING */
 
 
-int cm_path(char **buff, const char *file)
-    /** Create full pathname of config file */
+/**
+ *  Create full pathname of config file.
+ *
+ *  The resulting path is allocated within \a buff,
+ *  whose length is returned.
+ *
+ *  The path is calculated at runtime to allow
+ *  the built-in testing mechanisms to override the location
+ *  of configuration files via a command-line option,
+ *  when compiled with -DTESTING=1.
+ */
+int cm_path(char **buff, cm_path_prefix_t prefix_code, const char *file)
 {   size_t pfxlen, sfxlen;
-    const char *pfx=NULL;
+    const char *pfx = NULL;
 
     if (buff == NULL || file == NULL) return 0;
 
 #ifdef TESTING
-    pfx = (test_ctxtptr->argconfigdir != NULL ? test_ctxtptr->argconfigdir : "/nowhere");
+    pfx = (test_ctxtptr->argconfigdir != NULL ? test_ctxtptr->argconfigdir
+                                              : "/nowhere");
 #else
-    pfx = CM_CONFIG_DIR;
+    switch (prefix_code) {
+      case CM_SYSCONF_PFX:
+        pfx = CM_SYSCONF_DIR;
+        break;
+      case CM_SYSRUN_PFX:
+        pfx = CM_SYSRUN_DIR;
+        break;
+      default:
+        pfx = CM_SYSCONF_DIR;
+        break;
+    }
 #endif
 
     pfxlen = strlen(pfx);
@@ -990,5 +1011,5 @@ int ut_pwfort()
 #endif  /* TESTING */
 
 /*
- *  (C)Copyright 2005-2013, RW Penney
+ *  (C)Copyright 2005-2014, RW Penney
  */
