@@ -1,6 +1,6 @@
 /*
  *  Declarations for miscellaneous utilities for cryptmount
- *  (C)Copyright 2005-2013, RW Penney
+ *  (C)Copyright 2005-2014, RW Penney
  */
 
 /*
@@ -48,6 +48,15 @@ typedef struct cm_string {
     size_t size;                /**< Current length of string (less null) */
 } cm_string_t;
 
+
+/**
+ *  Symbolic representations of directories containing
+ *  either configuration files (e.g. /etc/cryptmount/),
+ *  or run-time state files (e.g. /run).
+ */
+typedef enum {
+    CM_SYSCONF_PFX, CM_SYSRUN_PFX } cm_path_prefix_t;
+
 cm_string_t *cm_str_init(char *val);
 cm_string_t *cm_str_alloc(size_t bufflen);
 cm_string_t *cm_str_realloc(cm_string_t *str, size_t bufflen);
@@ -57,7 +66,7 @@ cm_string_t *cm_str_append_str(cm_string_t *str, const char *addend);
 char *cm_str_strip(cm_string_t *str);
 void cm_str_free(cm_string_t *str);
 
-int cm_path(char **buff, const char *file);
+int cm_path(char **buff, cm_path_prefix_t prefix, const char *file);
 char *cm_strdup(const char *orig);
 int cm_strcasecmp(const char *s1, const char *s2);
 int cm_startswith(const char **str, const char *prefix);
@@ -126,5 +135,5 @@ static inline int cm_fwrite(const void *buff, size_t nbytes, FILE *stream) {
 #endif  /* _UTILS_H */
 
 /*
- *  (C)Copyright 2005-2013, RW Penney
+ *  (C)Copyright 2005-2014, RW Penney
  */
